@@ -53,6 +53,7 @@ static gint verbose_topics = 0;
 static gboolean is_debugging = FALSE;
 static gboolean replace_current = FALSE;
 static int no_prefix = 0;
+static gboolean is_wayland_compositor = FALSE;
 
 #ifdef WITH_VERBOSE_MODE
 static FILE* logfile = NULL;
@@ -79,7 +80,7 @@ ensure_logfile (void)
       
       if (err != NULL)
         {
-          meta_warning (_("Failed to open debug log: %s\n"),
+          meta_warning ("Failed to open debug log: %s\n",
                         err->message);
           g_error_free (err);
           return;
@@ -89,13 +90,13 @@ ensure_logfile (void)
       
       if (logfile == NULL)
         {
-          meta_warning (_("Failed to fdopen() log file %s: %s\n"),
+          meta_warning ("Failed to fdopen() log file %s: %s\n",
                         filename, strerror (errno));
           close (fd);
         }
       else
         {
-          g_printerr (_("Opened log file %s\n"), filename);
+          g_printerr ("Opened log file %s\n", filename);
         }
       
       g_free (filename);
@@ -192,6 +193,18 @@ meta_set_replace_current_wm (gboolean setting)
   replace_current = setting;
 }
 
+gboolean
+meta_is_wayland_compositor (void)
+{
+  return is_wayland_compositor;
+}
+
+void
+meta_set_is_wayland_compositor (gboolean value)
+{
+  is_wayland_compositor = value;
+}
+
 char *
 meta_g_utf8_strndup (const gchar *src,
                      gsize        n)
@@ -259,7 +272,7 @@ meta_debug_spew_real (const char *format, ...)
   out = logfile ? logfile : stderr;
   
   if (no_prefix == 0)
-    utf8_fputs (_("Window manager: "), out);
+    utf8_fputs ("Window manager: ", out);
   utf8_fputs (str, out);
 
   fflush (out);
@@ -409,7 +422,7 @@ meta_bug (const char *format, ...)
 #endif
 
   if (no_prefix == 0)
-    utf8_fputs (_("Bug in window manager: "), out);
+    utf8_fputs ("Bug in window manager: ", out);
   utf8_fputs (str, out);
 
   fflush (out);
@@ -440,7 +453,7 @@ meta_warning (const char *format, ...)
 #endif
 
   if (no_prefix == 0)
-    utf8_fputs (_("Window manager warning: "), out);
+    utf8_fputs ("Window manager warning: ", out);
   utf8_fputs (str, out);
 
   fflush (out);
@@ -468,7 +481,7 @@ meta_fatal (const char *format, ...)
 #endif
 
   if (no_prefix == 0)
-    utf8_fputs (_("Window manager error: "), out);
+    utf8_fputs ("Window manager error: ", out);
   utf8_fputs (str, out);
 
   fflush (out);
@@ -566,23 +579,6 @@ meta_external_binding_name_for_action (guint keybinding_action)
   return g_strdup_printf ("external-grab-%u", keybinding_action);
 }
 
-static gboolean
-zenity_supports_option (const char *section, const char *option)
-{
-  char *command, *out;
-  gboolean rv;
-
-  command = g_strdup_printf ("zenity %s", section);
-  g_spawn_command_line_sync (command, &out, NULL, NULL, NULL);
-
-  rv = (out && strstr (out, option));
-
-  g_free (command);
-  g_free (out);
-
-  return rv;
-}
-
 /* Command line arguments are passed in the locale encoding; in almost
  * all cases, we'd hope that is UTF-8 and no conversion is necessary.
  * If it's not UTF-8, then it's possible that the message isn't
@@ -671,13 +667,8 @@ meta_show_dialog (const char *type,
 
   if (icon_name)
     {
-      char *option = g_strdup_printf ("--help%s", type + 1);
-      if (zenity_supports_option (option, "--icon-name"))
-        {
-          append_argument (args, "--icon-name");
-          append_argument (args, icon_name);
-        }
-      g_free (option);
+      append_argument (args, "--icon-name");
+      append_argument (args, icon_name);
     }
 
   tmp = columns;
@@ -701,8 +692,7 @@ meta_show_dialog (const char *type,
       setenv ("WINDOWID", env, 1);
       g_free (env);
 
-      if (zenity_supports_option ("--help-general", "--modal"))
-        append_argument (args, "--modal");
+      append_argument (args, "--modal");
     }
 
   g_ptr_array_add (args, NULL); /* NULL-terminate */
@@ -912,6 +902,7 @@ meta_later_add (MetaLaterType  when,
        * there so it will happen before GTK+ repaints.
        */
       later->source = g_idle_add_full (META_PRIORITY_RESIZE, call_idle_later, later, NULL);
+      g_source_set_name_by_id (later->source, "[mutter] call_idle_later");
       ensure_later_repaint_func ();
       break;
     case META_LATER_CALC_SHOWING:
@@ -922,6 +913,7 @@ meta_later_add (MetaLaterType  when,
       break;
     case META_LATER_IDLE:
       later->source = g_idle_add_full (G_PRIORITY_DEFAULT_IDLE, call_idle_later, later, NULL);
+      g_source_set_name_by_id (later->source, "[mutter] call_idle_later");
       break;
     }
 
