@@ -46,13 +46,6 @@ Display* meta_ui_get_display (void);
 
 gint meta_ui_get_screen_number (void);
 
-void meta_ui_add_event_func    (Display       *xdisplay,
-                                MetaEventFunc  func,
-                                gpointer       data);
-void meta_ui_remove_event_func (Display       *xdisplay,
-                                MetaEventFunc  func,
-                                gpointer       data);
-
 MetaUI* meta_ui_new (Display *xdisplay,
                      Screen  *screen);
 void    meta_ui_free (MetaUI *ui);
@@ -149,28 +142,9 @@ GdkPixbuf* meta_ui_get_default_mini_icon (MetaUI *ui);
 gboolean  meta_ui_window_should_not_cause_focus (Display *xdisplay,
                                                  Window   xwindow);
 
-char*     meta_text_property_to_utf8 (Display             *xdisplay,
-                                      const XTextProperty *prop);
-
 void     meta_ui_set_current_theme (const char *name);
 gboolean meta_ui_have_a_theme      (void);
 
-/* Not a real key symbol but means "key above the tab key"; this is
- * used as the default keybinding for cycle_group.
- * 0x2xxxxxxx is a range not used by GDK or X. the remaining digits are
- * randomly chosen */
-#define META_KEY_ABOVE_TAB 0x2f7259c9
-
-gboolean meta_ui_parse_accelerator (const char          *accel,
-                                    unsigned int        *keysym,
-                                    unsigned int        *keycode,
-                                    MetaVirtualModifier *mask);
-gboolean meta_ui_parse_modifier    (const char          *accel,
-                                    MetaVirtualModifier *mask);
-
-/* Caller responsible for freeing return string of meta_ui_accelerator_name! */
-gchar*   meta_ui_accelerator_name  (unsigned int        keysym,
-                                    MetaVirtualModifier mask);
 gboolean meta_ui_window_is_widget (MetaUI *ui,
                                    Window  xwindow);
 
